/*
 * Scriptographer script  Theme Colours  
 * @ http://scriptographer.org 
 *
 * Takes line and fill colour from selected objects ( path, compound, group, text )
 * and applies to a second selected set of objects
 * with randomisation and optional control of line/fill colours.
 *
 * The idea is to use the first group as virtual swatches so an illustration gets random but themed colours.
 * The first group need not be part of the final illustration - but deleted or off the artboard for instance.
 * 
 */

var objList; // the target array
var pathColours; // the source path colours
var fillColours; // the source fill colours
var allColours;  // merge the above two for easier logic later

// choice texts 
var useAny = 'any';
var useMatch = 'match';
var useSwap = 'swap';
var usePair = 'pair';  

// working variables and initial values
var setFills = true;
var setPaths = true;
var usage = useAny;

// Interface ---------------------------

var values = {
setFills: setFills, 
setPaths: setPaths,
usage: usage
};

var components = {
	setFills: {
		label: 'Set fills',
		type: 'boolean'
	},
	setPaths: {
		label: 'Set paths',
		type: 'boolean'
	},
	usage: { 
        type: 'list',
        label: 'Usage',
        options: [useAny, useMatch, useSwap, usePair]
	},
    Run: { 
        type: 'button',
        value:'Select',
        label: 'Sources',
        onClick: function() { LoadSources(); }
	},
    Source: { 
        type: 'button',
        value:'Select',
        label: 'Targets',
        onClick: function() { LoadTargets(); }
	},
    Target: { 
        type: 'button',
        value:'Run',
        label: 'Execute',
        onClick: function() {  Run(); }
    }
};

var palette = new Palette('Theme Colours', components, values);


// ---------------------------------------------------------


 function LoadParams() {
    // load parms for efficiency
	setFills = values.setFills; 
	setPaths = values.setPaths;
	usage = values.usage;
}

 function LoadSources () {
   
   // update the 2 arrays of the colours from the objects selected - going down to child level eventually - not in this version
   // this version not processing groups or text items
   var sources = document.getItems({type: [Path,CompoundPath], selected: true}); 
       if (! sources.length) {
           Dialog.alert('Select Objects for the colours');
           return false;
       } 
       
   pathColours = [];
   fillColours = [];
   allColours = [];
   bothColours = [];
   
   for (var i = 0 ; i < sources.length; i++)   {
      var obj = sources[i];
       
     addColour(pathColours, obj.strokeColor);
     addColour(allColours, obj.strokeColor);
     addColour(fillColours, obj.fillColor);
     addColour(allColours, obj.fillColor);
     addColour(bothColours, new Array(obj.strokeColor,obj.fillColor));

     
   } // loop
   
} // fn LoadSources


 function LoadTargets ( ) {
  // create array of the objects selected - going down to child level of groups
   objList = document.getItems({type: [Path,CompoundPath,TextItem], selected: true});
    if (! objList.length) {
        Dialog.alert('Select Objects to update');
        return false;
    } 

} //fn LoadTargets


 function Run() {

    LoadParams();

    var targetCount = objList.length;
    var pathColourCount = pathColours.length;
    var fillColourCount = fillColours.length;
    var allColourCount = allColours.length;
    var bothColourCount = bothColours.length;
    
    for (var i = 0 ; i < objList.length; i++) { 
    
       var obj = objList[i];
       var colourIdx;
       
       if (usage == useMatch) {
       
          if (setPaths) {
			  colourIdx =  Math.round( Math.random() * (pathColourCount - 1) );
			  setPath(obj, pathColours[colourIdx]);
          }
          if (setFills) {
			  colourIdx =  Math.round( Math.random() * (fillColourCount - 1) );		
			  setFill(obj, fillColours[colourIdx]);
          }
          
       } else if (usage == useSwap) {
       
          if (setPaths) {
			  colourIdx =  Math.round( Math.random() * (fillColourCount - 1) );
			  setPath(obj, fillColours[colourIdx]);
          }
          if (setFills) {
			  colourIdx =  Math.round( Math.random() * (pathColourCount - 1) );
			  setFill(obj, pathColours[colourIdx]);
          }
      } else if (usage == usePair)  {
	  		    colourIdx =  Math.round( Math.random() * (bothColourCount - 1) );                
	  			if (setPaths) {
	  			  setPath(obj, bothColours[colourIdx][0]);
	            }
	            if (setFills) {
	  			  setFill(obj, bothColours[colourIdx][1]);
	            }
       } else {  // useAny
       
          if (setPaths) {
			  colourIdx =  Math.round( Math.random() * (allColourCount - 1) );
			  setPath(obj, allColours[colourIdx]);
          }
          if (setFills) {
			  colourIdx =  Math.round( Math.random() * (allColourCount - 1) );
			  setFill(obj, allColours[colourIdx]);
          }
       }

    } // loop objects


} // fn Run  ---------------------------------------------



 function setFill(obj, fill) {

	if (fill != null && fill.toString().indexOf("Grad") != -1 ) {
	   // copy a gradient fill
	   
	   // calculate slope of gradient and scale the destination y values from that
	   var objB = obj.bounds;
	   var o1X = fill.origin.x;
	   var o1Y = fill.origin.y;
	   var d1X = fill.destination.x;
	   var d1Y = fill.destination.y;
	   var slope = (d1Y - o1Y) / (d1X - o1X);
	   var c2Y = objB.center.y;
	   var d2Y = c2Y - obj.bounds.width * slope / 2;
	   var o2Y = c2Y + obj.bounds.width * slope / 2;
	   
	   // if radial fill set origin to centre	   
	   if (fill.gradient.type == 'radial') {
	   	   obj.fillColor = new GradientColor(fill.gradient,  objB.center,  objB.rightCenter);
	   } else {
	       obj.fillColor = new GradientColor(fill.gradient, new Point(objB.left, o2Y), new Point(objB.right, d2Y));
	   }
		
	} else {
	   // plain or patterb fill
	   obj.fillColor = fill;

	}  

} // fn setFill

function setPath(obj, path) {

   var objB = obj.bounds;
   
	if (path != null && path.toString().indexOf("Grad") != -1 ) {
	   // copy a gradient to the path
	   // looks better without slope - but may happen in version 2!

	   // if radial fill set origin to centre
	   if (path.type == 'radial') {
	   	   obj.strokeColor = new GradientColor(path.gradient,  objB.center, new Point(objB.right, d2Y));
	   } else {
	       obj.strokeColor = new GradientColor(path.gradient, objB.leftCenter, objB.rightCenter);
	   }
		
	} else {

	   obj.strokeColor = path;

	}  

} // fn setPath


function addColour( array, colour) {

   for (var i = 0; i < array.length; i++) {
		if (array == bothColours) { 
		   // check both colours
		   if (  CompareColours(colour[0], array[i][0]) && CompareColours(colour[1], array[i][1]) ) return;
		} else {
		   if (  CompareColours(colour, array[i]) )  return;
		}
    }	
	// not found so add it
	
	array.push(colour);

} // fn addColour



function CompareColours(c1, c2){

	// true if colours equal 
	// in type and array of components etc
	// Grad and pattern just compare for ident
	
	if (c1==null || c2==null) return;
	
  	function checkComponents(c1, c2, n ) {  //---------------
	// return true if n elements of two arrays are equal
       for (var idx = 0; idx < n; idx++) {
           if (c1[idx] != c2[idx]) { return false; }
       }  
       return true;
       
    }//---------------
  
    if (c1.toString().indexOf("Grad") != -1 && c2.toString().indexOf("Grad") == -1) {
       return (c1 == c2)
    }
    
    if (c1.toString().indexOf("Patt") != -1 && c2.toString().indexOf("Patt") == -1) {
       return (c1 == c2)
    }
    
    // now into normal colours 
	if (c1.type != c2.type) { return false}

    // now have the same type and need to check components
	// types are ('rgb', 'cmyk', 'gray', 'bitmap', 'argb', 'acmyk', 'agray', 'abitmap') 


    // if its a gradient or a pattern just compare idents
    // otherwise compare as follows

	switch(c1.type) {

		case 'rgb' :
		   return checkComponents(c1.components, c2.components,3) ? true : false ;
		case 'cmyk':
		   return checkComponents(c1.components, c2.components,4) ? true : false ;
		case 'gray':
		   return checkComponents(c1.components, c2.components,1) ? true : false ;
		case 'bitmap':
		   return false;
		case 'argb':
		   return checkComponents(c1.components, c2.components,4) ? true : false ;
		case 'acmyk':
		   return checkComponents(c1.components, c2.components,5) ? true : false ;
		case 'agray':
		   return checkComponents(c1.components, c2.components,2) ? true : false ;
		case 'abitmap':
		   return false;

	} // switch type

} // fn CompareColours

